package ru.dbotthepony.mc.otm.datagen.advancements

import net.minecraft.advancements.Advancement
import net.minecraft.advancements.AdvancementRewards
import net.minecraft.advancements.FrameType
import net.minecraft.advancements.RequirementsStrategy
import net.minecraft.advancements.critereon.DamagePredicate
import net.minecraft.advancements.critereon.DamageSourcePredicate
import net.minecraft.advancements.critereon.EntityPredicate
import net.minecraft.advancements.critereon.InventoryChangeTrigger
import net.minecraft.advancements.critereon.ItemPredicate
import net.minecraft.advancements.critereon.MinMaxBounds.Doubles
import net.minecraft.advancements.critereon.PlayerHurtEntityTrigger
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.entity.EntityType
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraftforge.common.data.ExistingFileHelper
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.datagen.lang.MatteryLanguageProvider
import ru.dbotthepony.mc.otm.datagen.modLocation
import ru.dbotthepony.mc.otm.registry.MItemTags
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.triggers.AndroidBatteryTrigger
import ru.dbotthepony.mc.otm.triggers.AndroidResearchTrigger
import ru.dbotthepony.mc.otm.triggers.BecomeAndroidDeathTrigger
import ru.dbotthepony.mc.otm.triggers.BecomeAndroidSleepTrigger
import ru.dbotthepony.mc.otm.triggers.BecomeAndroidTrigger
import ru.dbotthepony.mc.otm.triggers.BecomeHumaneTrigger
import ru.dbotthepony.mc.otm.triggers.EnderTeleporterFallDeathTrigger
import ru.dbotthepony.mc.otm.triggers.FallDampenersSaveTrigger
import ru.dbotthepony.mc.otm.triggers.NanobotsArmorTrigger
import ru.dbotthepony.mc.otm.triggers.PhantomSpawnDeniedTrigger
import ru.dbotthepony.mc.otm.triggers.ShockwaveDamageMobTrigger
import ru.dbotthepony.mc.otm.triggers.ShockwaveTrigger
import java.util.function.Consumer

fun addAndroidAdvancements(serializer: Consumer<Advancement>, existingFileHelper: ExistingFileHelper, lang: MatteryLanguageProvider) {
	val translation = lang.english.Prepended("otm.advancements.android")

	val root = AdvancementBuilder()
		.display(
			itemStack = ItemStack(MItems.PILL_ANDROID),
			title = TranslatableComponent(translation.add("root", "Androids and Humans")),
			description = TranslatableComponent(translation.add("root.desc", "Can you make out who is cruel machine and who can love?")),
			showToast = false,
			announceChat = false,
			background = modLocation("textures/block/decorative/metal_beam_top.png")
		)
		.addCriterion("became_android", BecomeAndroidTrigger.Instance)
		.save(serializer, modLocation("android/root"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.ZPM_BATTERY),
			title = TranslatableComponent(translation.add("zpm", "Fully Autonomous")),
			description = TranslatableComponent(translation.add("zpm.desc", "Use Zero Point Module as internal battery power source. Only time is your enemy now")),
			hidden = true,
			frameType = FrameType.CHALLENGE
		)
		.addCriterion("item", AndroidBatteryTrigger.Instance(ItemPredicate.Builder.item().of(MItems.ZPM_BATTERY).build()))
		.save(serializer, modLocation("android/zpm"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.QUANTUM_BATTERY),
			title = TranslatableComponent(translation.add("quantum_battery", "Wireless Charged")),
			description = TranslatableComponent(translation.add("quantum_battery.desc", "Use Quantum Battery as internal battery power source, might as well have Fission Reactor charge other side of the link")),
			hidden = true,
			frameType = FrameType.GOAL
		)
		.requirements(RequirementsStrategy.OR)
		.addCriterion("item0", AndroidBatteryTrigger.Instance(ItemPredicate.Builder.item().of(MItems.QUANTUM_BATTERY).build()))
		.addCriterion("item1", AndroidBatteryTrigger.Instance(ItemPredicate.Builder.item().of(MItems.QUANTUM_CAPACITOR).build()))
		.save(serializer, modLocation("android/quantum_battery"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.PILL_ANDROID),
			title = TranslatableComponent(translation.add("normal", "Last Sweet Dreams")),
			description = TranslatableComponent(translation.add("normal.desc", "Become an Android in your dreams, a soulless machine... Or is it?")),
			hidden = true,
		)
		.addCriterion("became_android", BecomeAndroidSleepTrigger.Instance)
		.save(serializer, modLocation("android/become_thru_sleep"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.PILL_ANDROID),
			title = TranslatableComponent(translation.add("death", "The Old Way")),
			description = TranslatableComponent(translation.add("death.desc", "Become an Android in event of death, veteran's favorite")),
			hidden = true,
		)
		.addCriterion("became_android", BecomeAndroidDeathTrigger.Instance)
		.save(serializer, modLocation("android/become_thru_death"), existingFileHelper)

	AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.PILL_HUMANE),
			title = TranslatableComponent(translation.add("unandroid", "Feel Humane Again")),
			description = TranslatableComponent(translation.add("unandroid.desc", "Become fleshy after being a machine, yet something is still missing that you had before...")),
			hidden = true,
			frameType = FrameType.GOAL
		)
		.addCriterion("become_humane", BecomeHumaneTrigger.Instance)
		.save(serializer, modLocation("android/become_humane"), existingFileHelper)

	val phantoms = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(Items.PHANTOM_MEMBRANE),
			title = TranslatableComponent(translation.add("phantom_spawn_denied", "Insomnia Immune")),
			description = TranslatableComponent(translation.add("phantom_spawn_denied.desc", "Have Phantoms not come after you because you are not someone who needs to sleep")),
			hidden = true
		)
		.addCriterion("phantom_spawn_denied", PhantomSpawnDeniedTrigger.Instance)
		.save(serializer, modLocation("android/phantom_spawn_denied"), existingFileHelper)

	val attractor = AdvancementBuilder()
		.parent(phantoms)
		.display(
			itemStack = ItemStack(MItems.PHANTOM_ATTRACTOR),
			title = TranslatableComponent(translation.add("phantom_attractor", "Eversleeping Decoy")),
			description = TranslatableComponent(translation.add("phantom_attractor.desc", "Put together a Phantom Attractor, to be able to fight Phantoms as Android again")),
		)
		.addCriterion("has_item", criterion(MItems.PHANTOM_ATTRACTOR))
		.save(serializer, modLocation("regular/phantom_attractor"), existingFileHelper)

	AdvancementBuilder()
		.parent(attractor)
		.display(
			itemStack = ItemStack(MItems.PHANTOM_ATTRACTOR),
			title = TranslatableComponent(translation.add("phantom_attractor_research", "Deception of Phantoms")),
			description = TranslatableComponent(translation.add("phantom_attractor_research.desc", "Research into how to attract Phantoms the same way as the ones who need to sleep")),
		)
		.addCriterion("researched", AndroidResearchTrigger.Instance(modLocation(MNames.PHANTOM_ATTRACTOR)))
		.save(serializer, modLocation("regular/phantom_attractor_research"), existingFileHelper)

	val researchAnything = AdvancementBuilder()
		.parent(root)
		.display(
			itemStack = ItemStack(MItems.ANDROID_STATION),
			title = TranslatableComponent(translation.add("research_anything", "New Trick")),
			description = TranslatableComponent(translation.add("research_anything.desc", "Research anything as Android")),
		)
		.addCriterion("research_anything", AndroidResearchTrigger.Instance(null))
		.save(serializer, modLocation("android/research_anything"), existingFileHelper)

	AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(Items.WATER_BUCKET),
			title = TranslatableComponent(translation.add("air_bags", "Patching Up Wooden Vessel")),
			description = TranslatableComponent(translation.add("air_bags.desc", "Research Air Bags as Android, to float in water")),
		)
		.addCriterion("air_bags", AndroidResearchTrigger.Instance(modLocation(MNames.AIR_BAGS)))
		.save(serializer, modLocation("android/research_air_bags"), existingFileHelper)

	AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(Items.ENDER_EYE),
			title = TranslatableComponent(translation.add("night_vision", "Second Pair of Eyes")),
			description = TranslatableComponent(translation.add("night_vision.desc", "Research Night Vision as Android, to see in the dark")),
		)
		.addCriterion("night_vision", AndroidResearchTrigger.Instance(modLocation(MNames.NIGHT_VISION)))
		.save(serializer, modLocation("android/research_night_vision"), existingFileHelper)

	val nanobots = AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(MItems.MATTER_TRANSFORM_MATRIX),
			title = TranslatableComponent(translation.add("nanobots", "Nanomachines, Son!")),
			description = TranslatableComponent(translation.add("nanobots.desc", "Research Nanobots as Android, to unlock potent research that come after it")),
			hidden = true
		)
		.addCriterion("nanobots", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS)))
		.save(serializer, modLocation("android/research_nanobots"), existingFileHelper)

	val shielding = AdvancementBuilder()
		.parent(nanobots)
		.display(
			itemStack = ItemStack(Items.SHIELD),
			title = TranslatableComponent(translation.add("nanobots_armor_deflect", "Like a Concrete Wall")),
			description = TranslatableComponent(translation.add("nanobots_armor_deflect.desc", "Have Nanobots absorb more than 5 hears of damage while you are left still functioning")),
			hidden = true,
			frameType = FrameType.GOAL
		)
		.addCriterion("damage", NanobotsArmorTrigger.Instance(Doubles.atLeast(10.0)))
		.save(serializer, modLocation("android/nanobots_armor_deflect"), existingFileHelper)

	AdvancementBuilder()
		.parent(shielding)
		.display(
			itemStack = ItemStack(Items.SHIELD),
			title = TranslatableComponent(translation.add("nanobots_armor_deflect2", "Unstoppable Force vs Immovable Object")),
			description = TranslatableComponent(translation.add("nanobots_armor_deflect2.desc", "Have Nanobots absorb more than 10 hears of damage while you are left still functioning")),
			hidden = true,
			frameType = FrameType.CHALLENGE
		)
		.addCriterion("damage", NanobotsArmorTrigger.Instance(Doubles.atLeast(20.0)))
		.save(serializer, modLocation("android/nanobots_armor_deflect2"), existingFileHelper)

	AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(Items.FEATHER),
			title = TranslatableComponent(translation.add("fall_dampeners_save", "Lucky Landing")),
			description = TranslatableComponent(translation.add("fall_dampeners_save.desc", "Survive fall that would have otherwise be fatal without Fall Dampeners research")),
			frameType = FrameType.GOAL
		)
		.addCriterion("saved", FallDampenersSaveTrigger.Instance)
		.save(serializer, modLocation("android/fall_dampeners_save"), existingFileHelper)

	AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(Items.SKELETON_SKULL),
			title = TranslatableComponent(translation.add("ender_teleport_fall_death", "Navigation Error")),
			description = TranslatableComponent(translation.add("ender_teleport_fall_death.desc", "Fall to your death moments after Teleporting")),
			frameType = FrameType.GOAL,
			hidden = true
		)
		.addCriterion("death", EnderTeleporterFallDeathTrigger.Instance)
		.save(serializer, modLocation("android/ender_teleport_fall_death"), existingFileHelper)

	val regen = AdvancementBuilder()
		.parent(nanobots)
		.display(
			itemStack = ItemStack(Items.GOLDEN_APPLE),
			title = TranslatableComponent(translation.add("regen", "Field Repair Done Easy")),
			description = TranslatableComponent(translation.add("regen.desc", "Research Nanobots Regeneration as Android")),
		)
		.addCriterion("regen0", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS_REGENERATION_1)))
		.save(serializer, modLocation("android/regen"), existingFileHelper)

	AdvancementBuilder()
		.parent(regen)
		.display(
			itemStack = ItemStack(Items.ENCHANTED_GOLDEN_APPLE),
			title = TranslatableComponent(translation.add("regen_all", "Field Repair Done Effortless")),
			description = TranslatableComponent(translation.add("regen_all.desc", "Max out Nanobots Regeneration research")),
			frameType = FrameType.GOAL,
		)
		.addCriterion("regen0", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS_REGENERATION_1)))
		.addCriterion("regen1", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS_REGENERATION_2)))
		.addCriterion("regen2", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS_REGENERATION_3)))
		.addCriterion("regen3", AndroidResearchTrigger.Instance(modLocation(MNames.NANOBOTS_REGENERATION_4)))
		.save(serializer, modLocation("android/regen_all"), existingFileHelper)

	AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(MItems.ANDROID_STATION),
			title = TranslatableComponent(translation.add("research_all", "Mecha-agnomination")),
			description = TranslatableComponent(translation.add("research_all.desc", "Research everything (that don't block any other research)")),
			frameType = FrameType.CHALLENGE
		)
		.rewards(AdvancementRewards.Builder.experience(400).addLootTable(modLocation("research_all_android")))
		.also { advancement ->
			DataGen.researchProvider.generatedView.stream()
				.filter { it.allBlockedBy.isEmpty() && it.allBlocking.isEmpty() }
				.forEach {
					advancement.addCriterion(it.id.toString(), AndroidResearchTrigger.Instance(it))
				}
		}
		.save(serializer, modLocation("android/research_everything"), existingFileHelper)

	val shockwave = AdvancementBuilder()
		.parent(researchAnything)
		.display(
			itemStack = ItemStack(Items.PISTON),
			title = TranslatableComponent(translation.add("shockwave", "Supersonic Landing")),
			description = TranslatableComponent(translation.add("shockwave.desc", "Perform a Shockwave upon landing")),
		)
		.addCriterion("shockwave", ShockwaveTrigger.Instance)
		.save(serializer, modLocation("android/shockwave"), existingFileHelper)

	AdvancementBuilder()
		.parent(shockwave)
		.display(
			itemStack = ItemStack(Items.WARDEN_SPAWN_EGG),
			title = TranslatableComponent(translation.add("shockwave_warden", "Music To Their Ears")),
			description = TranslatableComponent(translation.add("shockwave_warden.desc", "Hurt Warden using Shockwave ability")),
			frameType = FrameType.GOAL
		)
		.addCriterion("shockwave_warden", ShockwaveDamageMobTrigger.Instance(EntityPredicate.Builder.entity().of(EntityType.WARDEN).build().wrap()))
		.save(serializer, modLocation("android/shockwave_warden"), existingFileHelper)
}
