package ru.dbotthepony.mc.otm.block.entity.storage

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.ServerConfig
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.block.entity.MatteryPoweredBlockEntity
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.capability.transferInner
import ru.dbotthepony.mc.otm.core.Decimal
import ru.dbotthepony.mc.otm.graph.storage.BasicStorageGraphNode
import ru.dbotthepony.mc.otm.graph.storage.StorageNetworkGraph
import ru.dbotthepony.mc.otm.menu.StoragePowerSupplierMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MNames
import ru.dbotthepony.mc.otm.core.getDecimal
import ru.dbotthepony.mc.otm.core.set

class StoragePowerSupplierBlockEntity(blockPos: BlockPos, blockState: BlockState) : MatteryPoweredBlockEntity(MBlockEntities.STORAGE_POWER_SUPPLIER, blockPos, blockState) {
	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return StoragePowerSupplierMenu(containerID, inventory, this)
	}

	val cell = BasicStorageGraphNode()

	var powerPassed = Decimal.ZERO
		private set

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel) {
			StorageNetworkGraph.discoverFull(this, cell.storageNode)
		}
	}

	private var valid = true

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		return if (valid && cap === MatteryCapability.STORAGE_NODE) {
			cell.get().cast()
		} else super.getCapability(cap, side)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		cell.invalidate()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		cell.revive()
		valid = true
	}

	override fun setRemoved() {
		super.setRemoved()
		cell.destroy(level)
		valid = false
	}

	fun tick() {
		batteryChargeLoop()

		if (energy.batteryLevel.isZero)
			return

		val graph = cell.storageGraph ?: return

		if (graph.powerDemandingNodes.isEmpty())
			return

		var demand = Decimal.ZERO
		var i = 0

		val available = energy.batteryLevel.coerceAtMost(ServerConfig.STORAGE_POWER_SUPPLIER.throughput)

		for (demanding in graph.powerDemandingNodes) {
			val received = demanding.receiveEnergyOuter(available, true)

			if (received.isPositive) {
				demand += received
				i++
			}
		}

		if (demand.isZero) {
			return
		} else if (demand < available) {
			for (demanding in graph.powerDemandingNodes) {
				powerPassed += energy.transferInner(demanding, available, false)
			}
		} else {
			val forEach = available / i

			for (demanding in graph.powerDemandingNodes) {
				powerPassed += energy.transferInner(demanding, forEach, false)
			}
		}
	}

	override val energy = WorkerEnergyStorage(this, ServerConfig.STORAGE_POWER_SUPPLIER)

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[POWER_PASSED_KEY] = powerPassed.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		powerPassed = nbt.getDecimal(POWER_PASSED_KEY)
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.${OverdriveThatMatters.MOD_ID}.${MNames.STORAGE_POWER_SUPPLIER}")
		const val POWER_PASSED_KEY = "powerPassed"
	}
}
