package ru.dbotthepony.mc.otm.client.render

import net.minecraft.resources.ResourceLocation

class SubSkinElement(
	val parent: AbstractSkinElement,
	val xOffset: Float = 0f,
	val yOffset: Float = 0f,
	override val width: Float = parent.width,
	override val height: Float = parent.height,
	private val overrideWinding: UVWindingOrder? = null
) : AbstractSkinElement.Mutable() {
	override var u0: Float = 0f
		private set
	override var v0: Float = 0f
		private set
	override var u1: Float = 0f
		private set
	override var v1: Float = 0f
		private set

	private fun calculateUVs() {
		u0 = parent.u0 + (xOffset / parent.width) * (parent.u1 - parent.u0)
		v0 = parent.v0 + (yOffset / parent.height) * (parent.v1 - parent.v0)
		u1 = parent.u0 + ((xOffset + width) / parent.width) * (parent.u1 - parent.u0)
		v1 = parent.v0 + ((yOffset + height) / parent.height) * (parent.v1 - parent.v0)

		notifyListeners()
	}

	override fun parentChanges(parent: AbstractSkinElement) = calculateUVs()

	init {
		parent.addListener(this)
	}

	init {
		calculateUVs()
	}

	override val texture: ResourceLocation
		get() = parent.texture

	override val type: SkinElementType
		get() = SkinElementType.SUBELEMENT

	override val winding: UVWindingOrder
		get() = overrideWinding ?: parent.winding

	fun copy(
		xOffset: Float = this.xOffset,
		yOffset: Float = this.yOffset,
		width: Float = this.width,
		height: Float = this.height,
		overrideWinding: UVWindingOrder? = this.overrideWinding
	) = SubSkinElement(parent, xOffset, yOffset, width, height, overrideWinding)
}

fun AbstractSkinElement.subElement(
	x: Float = 0f,
	y: Float = 0f,
	width: Float = this.width,
	height: Float = this.height,
) = SubSkinElement(this, x, y, width, height)
