package ru.dbotthepony.mc.otm.matter

import com.google.gson.JsonObject
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.TagKey
import net.minecraft.world.item.Item
import ru.dbotthepony.mc.otm.core.set

open class DeleteAction : AbstractRegistryAction {
	constructor(
		tag: TagKey<Item>,
		errorOnFailure: Boolean = false,
	) : super(tag = tag, errorOnFailure = errorOnFailure, matter = null, complexity = null)

	constructor(
		key: ResourceLocation,
		errorOnFailure: Boolean = false,
	) : super(key = key, errorOnFailure = errorOnFailure, matter = null, complexity = null)

	constructor(json: JsonObject) : super(json)

	override fun toJson(): JsonObject {
		return super.toJson().also {
			it["action"] = "delete"
		}
	}

	override fun update(registry: MutableCollection<MutableEntry>, modifier: ResourceLocation) {
		if (!checkConditions()) {
			return
		}

		check(matter == null) { "Delete action can't have matter value (for $modifier)" }
		check(complexity == null) { "Delete action can't have complexity value (for $modifier)" }

		if (key != null) {
			val iterator = registry.iterator()

			for (value in iterator) {
				if (value is MutableKeyEntry && value.key == key) {
					iterator.remove()
					return
				}
			}

			fail { "Could not find matter value with key $key" }
		} else {
			val iterator = registry.iterator()

			for (value in iterator) {
				if (value is MutableTagEntry && value.tag == tag) {
					iterator.remove()
					return
				}
			}

			fail { "Could not find matter value with tag $tag" }
		}
	}
}
