package ru.dbotthepony.mc.otm.menu

import net.minecraft.core.Direction
import kotlin.jvm.JvmOverloads
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.inventory.Slot
import net.minecraft.world.level.block.Block
import ru.dbotthepony.mc.otm.block.EnergyCounterBlock
import ru.dbotthepony.mc.otm.block.entity.EnergyCounterBlockEntity
import ru.dbotthepony.mc.otm.core.Decimal
import ru.dbotthepony.mc.otm.menu.widget.NumberPlayerInputWidget
import ru.dbotthepony.mc.otm.menu.widget.OneWayPlayerInputWidget
import ru.dbotthepony.mc.otm.registry.MMenus
import java.math.BigDecimal

class EnergyCounterMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: EnergyCounterBlockEntity? = null
) : MatteryMenu(MMenus.ENERGY_COUNTER, p_38852_, inventory, tile) {
	var passed by mSynchronizer.fraction()
	var average by mSynchronizer.fraction()
	var last20Ticks by mSynchronizer.fraction()
	var lastTick by mSynchronizer.fraction()

	val switchDirection: OneWayPlayerInputWidget
	var inputDirection by mSynchronizer.enum(Direction::class.java)

	val maxIO = NumberPlayerInputWidget(this)

	// TODO: Graph and proper networking for it
	private var ticksPassed = 0

	init {
		if (tile == null) {
			switchDirection = OneWayPlayerInputWidget(this)
			maxIO.asClient()
		} else {
			switchDirection = OneWayPlayerInputWidget(this) {
				tile.level?.setBlock(tile.blockPos, tile.blockState.setValue(EnergyCounterBlock.INPUT_DIRECTION, tile.blockState.getValue(EnergyCounterBlock.INPUT_DIRECTION).opposite), Block.UPDATE_ALL)
			}

			maxIO.withSupplier {
				return@withSupplier if (tile.ioLimit == null) MINUS_ONE else tile.ioLimit!!.toBigDecmial()
			}.withConsumer {
				if (it.signum() < 0) {
					tile.ioLimit = null
				} else {
					tile.ioLimit = Decimal(it)
				}
			}
		}
	}

	override fun broadcastChanges() {
		if (tile is EnergyCounterBlockEntity) {
			passed = tile.passed
			average = tile.calcAverage(20)
			lastTick = tile.lastTick

			if (ticksPassed == 0) {
				last20Ticks = tile.sumHistory(20)
			}

			ticksPassed = (ticksPassed + 1) % 20
			inputDirection = tile.blockState.getValue(EnergyCounterBlock.INPUT_DIRECTION)
		}

		super.broadcastChanges()
	}


	override val storageSlots: Collection<Slot> get() = emptySet()

	companion object {
		private val MINUS_ONE = -BigDecimal.ONE
	}
}
