package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.google.gson.JsonSyntaxException
import net.minecraft.advancements.critereon.AbstractCriterionTriggerInstance
import net.minecraft.advancements.critereon.DamagePredicate
import net.minecraft.advancements.critereon.DamageSourcePredicate
import net.minecraft.advancements.critereon.DeserializationContext
import net.minecraft.advancements.critereon.EntityPredicate
import net.minecraft.advancements.critereon.MinMaxBounds
import net.minecraft.advancements.critereon.SerializationContext
import net.minecraft.advancements.critereon.SimpleCriterionTrigger
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.damagesource.DamageSource
import net.minecraft.world.entity.Entity
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.set

object ShockwaveDamageMobTrigger: SimpleCriterionTrigger<ShockwaveDamageMobTrigger.Instance>() {
	val ID = ResourceLocation(OverdriveThatMatters.MOD_ID, "shockwave_damage_mob")

	override fun getId(): ResourceLocation {
		return ID
	}

	override fun createInstance(
		p_66248_: JsonObject,
		p_66249_: EntityPredicate.Composite,
		p_66250_: DeserializationContext
	): Instance {
		return Instance(
			EntityPredicate.Composite.fromJson(p_66248_, "entity_predicate", p_66250_),
			(p_66248_["damage"] as? JsonObject)?.let(DamagePredicate::fromJson) ?: DamagePredicate.ANY
		)
	}

	fun trigger(player: ServerPlayer, entity: Entity, damageSource: DamageSource, damage: Float) {
		val context = EntityPredicate.createContext(player, entity)

		trigger(player) {
			it.predicate.matches(context) && it.damagePredicate.matches(player, damageSource, damage, damage, false)
		}
	}

	class Instance(
		val predicate: EntityPredicate.Composite = EntityPredicate.Composite.ANY,
		val damagePredicate: DamagePredicate = DamagePredicate(
			MinMaxBounds.Doubles.atLeast(1.0),
			MinMaxBounds.Doubles.atLeast(1.0),
			EntityPredicate.ANY,
			null,
			DamageSourcePredicate.ANY
		)
	) : AbstractCriterionTriggerInstance(ID, EntityPredicate.Composite.ANY) {
		override fun serializeToJson(p_16979_: SerializationContext): JsonObject {
			return super.serializeToJson(p_16979_).also {
				it["entity_predicate"] = predicate.toJson(p_16979_)
				it["damage"] = damagePredicate.serializeToJson()
			}
		}
	}
}
